@@ -57,10 +57,10 @@ __all__ = ['Path', 'TempDir']
 
 LINESEPS = ['\r\n', '\r', '\n']
 U_LINESEPS = LINESEPS + ['\u0085', '\u2028', '\u2029']
-NEWLINE = re.compile('|'.join(LINESEPS))
 U_NEWLINE = re.compile('|'.join(U_LINESEPS))
-NL_END = re.compile(r'(?:{0})$'.format(NEWLINE.pattern))
-U_NL_END = re.compile(r'(?:{0})$'.format(U_NEWLINE.pattern))
+B_NEWLINE = re.compile('|'.join(LINESEPS).encode())
+B_NL_END = re.compile(f'(?:{B_NEWLINE.pattern.decode()})$'.encode())
+U_NL_END = re.compile(f'(?:{U_NEWLINE.pattern})$')
 
 
 class TreeWalkWarning(Warning):
@@ -182,14 +182,9 @@ class Path(str):
 
     # Adding a Path and a string yields a Path.
     def __add__(self, more):
-        try:
-            return self._next_class(super(Path, self).__add__(more))
-        except TypeError:  # Python bug
-            return NotImplemented
+        return self._next_class(super(Path, self).__add__(more))
 
     def __radd__(self, other):
-        if not isinstance(other, str):
-            return NotImplemented
         return self._next_class(other.__add__(self))
 
     # The / operator joins Paths.
@@ -228,14 +223,11 @@ class Path(str):
     def __exit__(self, *_):
         os.chdir(self._old_dir)
 
-    def __fspath__(self):
-        return self
-
     @classmethod
     def getcwd(cls):
         """Return the current working directory as a path object.
 
-        .. seealso:: :func:`os.getcwdu`
+        .. seealso:: :func:`os.getcwd`
         """
         return cls(os.getcwd())
 
@@ -243,36 +235,36 @@ class Path(str):
     # --- Operations on Path strings.
 
     def abspath(self):
-        """ .. seealso:: :func:`os.path.abspath` """
+        """.. seealso:: :func:`os.path.abspath`"""
         return self._next_class(self.module.abspath(self))
 
     def normcase(self):
-        """ .. seealso:: :func:`os.path.normcase` """
+        """.. seealso:: :func:`os.path.normcase`"""
         return self._next_class(self.module.normcase(self))
 
     def normpath(self):
-        """ .. seealso:: :func:`os.path.normpath` """
+        """.. seealso:: :func:`os.path.normpath`"""
         return self._next_class(self.module.normpath(self))
 
     def realpath(self):
-        """ .. seealso:: :func:`os.path.realpath` """
+        """.. seealso:: :func:`os.path.realpath`"""
         realpath = best_realpath(self.module)
         return self._next_class(realpath(self))
 
     def expanduser(self):
-        """ .. seealso:: :func:`os.path.expanduser` """
+        """.. seealso:: :func:`os.path.expanduser`"""
         return self._next_class(self.module.expanduser(self))
 
     def expandvars(self):
-        """ .. seealso:: :func:`os.path.expandvars` """
+        """.. seealso:: :func:`os.path.expandvars`"""
         return self._next_class(self.module.expandvars(self))
 
     def dirname(self):
-        """ .. seealso:: :attr:`parent`, :func:`os.path.dirname` """
+        """.. seealso:: :attr:`parent`, :func:`os.path.dirname`"""
         return self._next_class(self.module.dirname(self))
 
     def basename(self):
-        """ .. seealso:: :attr:`name`, :func:`os.path.basename` """
+        """.. seealso:: :attr:`name`, :func:`os.path.basename`"""
         return self._next_class(self.module.basename(self))
 
     def expand(self):
@@ -296,7 +288,7 @@ class Path(str):
 
     @property
     def ext(self):
-        """ The file extension, for example ``'.py'``. """
+        """The file extension, for example ``'.py'``."""
         f, ext = self.module.splitext(self)
         return ext
 
@@ -356,7 +348,7 @@ class Path(str):
     )
 
     def splitpath(self):
-        """p.splitpath() -> Return ``(p.parent, p.name)``.
+        """Return two-tuple of ``.parent``, ``.name``.
 
         .. seealso:: :attr:`parent`, :attr:`name`, :func:`os.path.split`
         """
@@ -364,7 +356,7 @@ class Path(str):
         return self._next_class(parent), child
 
     def splitdrive(self):
-        """p.splitdrive() -> Return ``(p.drive, <the rest of p>)``.
+        """Return two-tuple of ``.drive`` and rest without drive.
 
         Split the drive specifier from this path.  If there is
         no drive specifier, :samp:`{p.drive}` is empty, so the return value
@@ -373,10 +365,10 @@ class Path(str):
         .. seealso:: :func:`os.path.splitdrive`
         """
         drive, rel = self.module.splitdrive(self)
-        return self._next_class(drive), rel
+        return self._next_class(drive), self._next_class(rel)
 
     def splitext(self):
-        """p.splitext() -> Return ``(p.stripext(), p.ext)``.
+        """Return two-tuple of ``.stripext()`` and ``.ext``.
 
         Split the filename extension from this path and return
         the two parts.  Either part may be empty.
@@ -391,27 +383,13 @@ class Path(str):
         return self._next_class(filename), ext
 
     def stripext(self):
-        """p.stripext() -> Remove one file extension from the path.
+        """Remove one file extension from the path.
 
         For example, ``Path('/home/guido/python.tar.gz').stripext()``
         returns ``Path('/home/guido/python.tar')``.
         """
         return self.splitext()[0]
 
-    def splitunc(self):
-        """ .. seealso:: :func:`os.path.splitunc` """
-        unc, rest = self.module.splitunc(self)
-        return self._next_class(unc), rest
-
-    @property
-    def uncshare(self):
-        """
-        The UNC mount point for this path.
-        This is empty for paths on local drives.
-        """
-        unc, r = self.module.splitunc(self)
-        return self._next_class(unc)
-
     @multimethod
     def joinpath(cls, first, *others):
         """
@@ -434,19 +412,32 @@ class Path(str):
         directory of this path (for example, ``'/'`` or ``'C:\\'``).  The
         other items in the list will be strings.
 
-        ``path.Path.joinpath(*result)`` will yield the original path.
+        ``Path.joinpath(*result)`` will yield the original path.
+
+        >>> Path('/foo/bar/baz').splitall()
+        [Path('/'), 'foo', 'bar', 'baz']
         """
-        parts = []
+        return list(self._parts())
+
+    def parts(self):
+        """
+        >>> Path('/foo/bar/baz').parts()
+        (Path('/'), 'foo', 'bar', 'baz')
+        """
+        return tuple(self._parts())
+
+    def _parts(self):
+        return reversed(tuple(self._parts_iter()))
+
+    def _parts_iter(self):
         loc = self
         while loc != os.curdir and loc != os.pardir:
             prev = loc
             loc, child = prev.splitpath()
             if loc == prev:
                 break
-            parts.append(child)
-        parts.append(loc)
-        parts.reverse()
-        return parts
+            yield child
+        yield loc
 
     def relpath(self, start='.'):
         """Return this path as a relative path,
@@ -496,7 +487,7 @@ class Path(str):
     # --- Listing, searching, walking, and matching
 
     def listdir(self, match=None):
-        """D.listdir() -> List of items in this directory.
+        """List of items in this directory.
 
         Use :meth:`files` or :meth:`dirs` instead if you want a listing
         of just files or just subdirectories.
@@ -512,7 +503,7 @@ class Path(str):
         return list(filter(match, (self / child for child in os.listdir(self))))
 
     def dirs(self, *args, **kwargs):
-        """D.dirs() -> List of this directory's subdirectories.
+        """List of this directory's subdirectories.
 
         The elements of the list are Path objects.
         This does not walk recursively into subdirectories
@@ -523,7 +514,7 @@ class Path(str):
         return [p for p in self.listdir(*args, **kwargs) if p.isdir()]
 
     def files(self, *args, **kwargs):
-        """D.files() -> List of the files in this directory.
+        """List of the files in self.
 
         The elements of the list are Path objects.
         This does not walk into subdirectories (see :meth:`walkfiles`).
@@ -534,7 +525,7 @@ class Path(str):
         return [p for p in self.listdir(*args, **kwargs) if p.isfile()]
 
     def walk(self, match=None, errors='strict'):
-        """D.walk() -> iterator over files and subdirs, recursively.
+        """Iterator over files and subdirs, recursively.
 
         The iterator yields Path objects naming each child item of
         this directory and its descendants.  This requires that
@@ -568,17 +559,18 @@ class Path(str):
                 do_traverse = traverse()
             except Exception as exc:
                 errors(f"Unable to access '{child}': {exc}")
+                continue
 
             if do_traverse:
                 for item in child.walk(errors=errors, match=match):
                     yield item
 
     def walkdirs(self, *args, **kwargs):
-        """D.walkdirs() -> iterator over subdirs, recursively."""
+        """Iterator over subdirs, recursively."""
         return (item for item in self.walk(*args, **kwargs) if item.isdir())
 
     def walkfiles(self, *args, **kwargs):
-        """D.walkfiles() -> iterator over files in D, recursively."""
+        """Iterator over files, recursively."""
         return (item for item in self.walk(*args, **kwargs) if item.isfile())
 
     def fnmatch(self, pattern, normcase=None):
@@ -589,8 +581,8 @@ class Path(str):
             attribute, it is applied to the name and path prior to comparison.
 
         `normcase` - (optional) A function used to normalize the pattern and
-            filename before matching. Defaults to :meth:`self.module`, which
-            defaults to :meth:`os.path.normcase`.
+            filename before matching. Defaults to normcase from
+            ``self.module``, :func:`os.path.normcase`.
 
         .. seealso:: :func:`fnmatch.fnmatch`
         """
@@ -639,15 +631,15 @@ class Path(str):
     # --- Reading or writing an entire file at once.
 
     def open(self, *args, **kwargs):
-        """Open this file and return a corresponding :class:`file` object.
+        """Open this file and return a corresponding file object.
 
         Keyword arguments work as in :func:`io.open`.  If the file cannot be
-        opened, an :class:`~exceptions.OSError` is raised.
+        opened, an :class:`OSError` is raised.
         """
         return io.open(self, *args, **kwargs)
 
     def bytes(self):
-        """ Open this file, read all bytes, return them as a string. """
+        """Open this file, read all bytes, return them as a string."""
         with self.open('rb') as f:
             return f.read()
 
@@ -698,7 +690,7 @@ class Path(str):
             return f.read()
 
     def text(self, encoding=None, errors='strict'):
-        """Legacy function to read text.
+        r"""Legacy function to read text.
 
         Converts all newline sequences to ``\n``.
         """
@@ -777,7 +769,7 @@ class Path(str):
             text = text.encode(encoding or sys.getdefaultencoding(), errors)
         else:
             assert encoding is None
-            text = NEWLINE.sub(linesep, text)
+            text = B_NEWLINE.sub(linesep.encode(), text)
         self.write_bytes(text, append=append)
 
     def lines(self, encoding=None, errors='strict', retain=True):
@@ -826,7 +818,7 @@ class Path(str):
                 default is os.linesep, which is platform-dependent
                 (``'\r\n'`` on Windows, ``'\n'`` on Unix, etc.).
                 Specify ``None`` to write the lines as-is, like
-                :meth:`file.writelines`.
+                ``.writelines`` on a file object.
 
         Use the keyword argument ``append=True`` to append lines to the
         file.  The default is to overwrite the file.
@@ -843,7 +835,7 @@ class Path(str):
             for line in lines:
                 isUnicode = isinstance(line, str)
                 if linesep is not None:
-                    pattern = U_NL_END if isUnicode else NL_END
+                    pattern = U_NL_END if isUnicode else B_NL_END
                     line = pattern.sub('', line) + linesep
                 if isUnicode:
                     line = line.encode(encoding or sys.getdefaultencoding(), errors)
@@ -895,66 +887,78 @@ class Path(str):
     # bound. Playing it safe and wrapping them all in method calls.
 
     def isabs(self):
-        """ .. seealso:: :func:`os.path.isabs` """
+        """
+        >>> Path('.').isabs()
+        False
+
+        .. seealso:: :func:`os.path.isabs`
+        """
         return self.module.isabs(self)
 
     def exists(self):
-        """ .. seealso:: :func:`os.path.exists` """
+        """.. seealso:: :func:`os.path.exists`"""
         return self.module.exists(self)
 
     def isdir(self):
-        """ .. seealso:: :func:`os.path.isdir` """
+        """.. seealso:: :func:`os.path.isdir`"""
         return self.module.isdir(self)
 
     def isfile(self):
-        """ .. seealso:: :func:`os.path.isfile` """
+        """.. seealso:: :func:`os.path.isfile`"""
         return self.module.isfile(self)
 
     def islink(self):
-        """ .. seealso:: :func:`os.path.islink` """
+        """.. seealso:: :func:`os.path.islink`"""
         return self.module.islink(self)
 
     def ismount(self):
-        """ .. seealso:: :func:`os.path.ismount` """
+        """
+        >>> Path('.').ismount()
+        False
+
+        .. seealso:: :func:`os.path.ismount`
+        """
         return self.module.ismount(self)
 
     def samefile(self, other):
-        """ .. seealso:: :func:`os.path.samefile` """
-        if not hasattr(self.module, 'samefile'):
-            other = Path(other).realpath().normpath().normcase()
-            return self.realpath().normpath().normcase() == other
+        """.. seealso:: :func:`os.path.samefile`"""
         return self.module.samefile(self, other)
 
     def getatime(self):
-        """ .. seealso:: :attr:`atime`, :func:`os.path.getatime` """
+        """.. seealso:: :attr:`atime`, :func:`os.path.getatime`"""
         return self.module.getatime(self)
 
     atime = property(
         getatime,
         None,
         None,
-        """ Last access time of the file.
+        """
+        Last access time of the file.
+
+        >>> Path('.').atime > 0
+        True
 
         .. seealso:: :meth:`getatime`, :func:`os.path.getatime`
         """,
     )
 
     def getmtime(self):
-        """ .. seealso:: :attr:`mtime`, :func:`os.path.getmtime` """
+        """.. seealso:: :attr:`mtime`, :func:`os.path.getmtime`"""
         return self.module.getmtime(self)
 
     mtime = property(
         getmtime,
         None,
         None,
-        """ Last-modified time of the file.
+        """
+        Last modified time of the file.
 
         .. seealso:: :meth:`getmtime`, :func:`os.path.getmtime`
         """,
     )
 
     def getctime(self):
-        """ .. seealso:: :attr:`ctime`, :func:`os.path.getctime` """
+        """.. seealso:: :attr:`ctime`, :func:`os.path.getctime`"""
         return self.module.getctime(self)
 
     ctime = property(
@@ -968,7 +972,7 @@ class Path(str):
     )
 
     def getsize(self):
-        """ .. seealso:: :attr:`size`, :func:`os.path.getsize` """
+        """.. seealso:: :attr:`size`, :func:`os.path.getsize`"""
         return self.module.getsize(self)
 
     size = property(
@@ -981,38 +985,45 @@ class Path(str):
         """,
     )
 
-    if hasattr(os, 'access'):
+    def access(self, *args, **kwargs):
+        """
+        Return does the real user have access to this path.
 
-        def access(self, mode):
-            """Return ``True`` if current user has access to this path.
+        >>> Path('.').access(os.F_OK)
+        True
 
-            mode - One of the constants :data:`os.F_OK`, :data:`os.R_OK`,
-            :data:`os.W_OK`, :data:`os.X_OK`
-
-            .. seealso:: :func:`os.access`
-            """
-            return os.access(self, mode)
+        .. seealso:: :func:`os.access`
+        """
+        return os.access(self, *args, **kwargs)
 
     def stat(self):
-        """Perform a ``stat()`` system call on this path.
+        """
+        Perform a ``stat()`` system call on this path.
+
+        >>> Path('.').stat()
+        os.stat_result(...)
 
         .. seealso:: :meth:`lstat`, :func:`os.stat`
         """
         return os.stat(self)
 
     def lstat(self):
-        """Like :meth:`stat`, but do not follow symbolic links.
+        """
+        Like :meth:`stat`, but do not follow symbolic links.
+
+        >>> Path('.').lstat() == Path('.').stat()
+        True
 
         .. seealso:: :meth:`stat`, :func:`os.lstat`
         """
         return os.lstat(self)
 
-    def __get_owner_windows(self):
-        """
+    def __get_owner_windows(self):  # pragma: nocover
+        r"""
         Return the name of the owner of this file or directory. Follow
         symbolic links.
 
-        Return a name of the form ``r'DOMAIN\\User Name'``; may be a group.
+        Return a name of the form ``DOMAIN\User Name``; may be a group.
 
         .. seealso:: :attr:`owner`
         """
@@ -1023,7 +1034,7 @@ class Path(str):
         account, domain, typecode = win32security.LookupAccountSid(None, sid)
         return domain + '\\' + account
 
-    def __get_owner_unix(self):
+    def __get_owner_unix(self):  # pragma: nocover
         """
         Return the name of the owner of this file or directory. Follow
         symbolic links.
@@ -1033,15 +1044,16 @@ class Path(str):
         st = self.stat()
         return pwd.getpwuid(st.st_uid).pw_name
 
-    def __get_owner_not_implemented(self):
+    def __get_owner_not_implemented(self):  # pragma: nocover
         raise NotImplementedError("Ownership not available on this platform.")
 
-    if 'win32security' in globals():
-        get_owner = __get_owner_windows
-    elif 'pwd' in globals():
-        get_owner = __get_owner_unix
-    else:
-        get_owner = __get_owner_not_implemented
+    get_owner = (
+        __get_owner_windows
+        if 'win32security' in globals()
+        else __get_owner_unix
+        if 'pwd' in globals()
+        else __get_owner_not_implemented
+    )
 
     owner = property(
         get_owner,
@@ -1064,18 +1076,18 @@ class Path(str):
     if hasattr(os, 'pathconf'):
 
         def pathconf(self, name):
-            """ .. seealso:: :func:`os.pathconf` """
+            """.. seealso:: :func:`os.pathconf`"""
             return os.pathconf(self, name)
 
     #
     # --- Modifying operations on files and directories
 
-    def utime(self, times):
+    def utime(self, *args, **kwargs):
         """Set the access and modified times of this file.
 
         .. seealso:: :func:`os.utime`
         """
-        os.utime(self, times)
+        os.utime(self, *args, **kwargs)
         return self
 
     def chmod(self, mode):
@@ -1091,30 +1103,31 @@ class Path(str):
         os.chmod(self, mode)
         return self
 
-    def chown(self, uid=-1, gid=-1):
-        """
-        Change the owner and group by names rather than the uid or gid numbers.
+    if hasattr(os, 'chown'):
 
-        .. seealso:: :func:`os.chown`
-        """
-        if hasattr(os, 'chown'):
-            if 'pwd' in globals() and isinstance(uid, str):
-                uid = pwd.getpwnam(uid).pw_uid
-            if 'grp' in globals() and isinstance(gid, str):
-                gid = grp.getgrnam(gid).gr_gid
-            os.chown(self, uid, gid)
-        else:
-            msg = "Ownership not available on this platform."
-            raise NotImplementedError(msg)
-        return self
+        def chown(self, uid=-1, gid=-1):
+            """
+            Change the owner and group by names or numbers.
+
+            .. seealso:: :func:`os.chown`
+            """
+
+            def resolve_uid(uid):
+                return uid if isinstance(uid, int) else pwd.getpwnam(uid).pw_uid
+
+            def resolve_gid(gid):
+                return gid if isinstance(gid, int) else grp.getgrnam(gid).gr_gid
+
+            os.chown(self, resolve_uid(uid), resolve_gid(gid))
+            return self
 
     def rename(self, new):
-        """ .. seealso:: :func:`os.rename` """
+        """.. seealso:: :func:`os.rename`"""
         os.rename(self, new)
         return self._next_class(new)
 
     def renames(self, new):
-        """ .. seealso:: :func:`os.renames` """
+        """.. seealso:: :func:`os.renames`"""
         os.renames(self, new)
         return self._next_class(new)
 
@@ -1122,7 +1135,7 @@ class Path(str):
     # --- Create/delete operations on directories
 
     def mkdir(self, mode=0o777):
-        """ .. seealso:: :func:`os.mkdir` """
+        """.. seealso:: :func:`os.mkdir`"""
         os.mkdir(self, mode)
         return self
 
@@ -1134,7 +1147,7 @@ class Path(str):
         return self
 
     def makedirs(self, mode=0o777):
-        """ .. seealso:: :func:`os.makedirs` """
+        """.. seealso:: :func:`os.makedirs`"""
         os.makedirs(self, mode)
         return self
 
@@ -1146,7 +1159,7 @@ class Path(str):
         return self
 
     def rmdir(self):
-        """ .. seealso:: :func:`os.rmdir` """
+        """.. seealso:: :func:`os.rmdir`"""
         os.rmdir(self)
         return self
 
@@ -1160,7 +1173,7 @@ class Path(str):
         return self
 
     def removedirs(self):
-        """ .. seealso:: :func:`os.removedirs` """
+        """.. seealso:: :func:`os.removedirs`"""
         os.removedirs(self)
         return self
 
@@ -1184,7 +1197,7 @@ class Path(str):
         return self
 
     def remove(self):
-        """ .. seealso:: :func:`os.remove` """
+        """.. seealso:: :func:`os.remove`"""
         os.remove(self)
         return self
 
@@ -1195,67 +1208,50 @@ class Path(str):
             self.unlink()
         return self
 
-    def unlink(self):
-        """ .. seealso:: :func:`os.unlink` """
-        os.unlink(self)
-        return self
-
-    def unlink_p(self):
-        """Like :meth:`unlink`, but does not raise an exception if the
-        file does not exist."""
-        self.remove_p()
-        return self
+    unlink = remove
+    unlink_p = remove_p
 
     # --- Links
 
-    if hasattr(os, 'link'):
+    def link(self, newpath):
+        """Create a hard link at `newpath`, pointing to this file.
 
-        def link(self, newpath):
-            """Create a hard link at `newpath`, pointing to this file.
+        .. seealso:: :func:`os.link`
+        """
+        os.link(self, newpath)
+        return self._next_class(newpath)
 
-            .. seealso:: :func:`os.link`
-            """
-            os.link(self, newpath)
-            return self._next_class(newpath)
+    def symlink(self, newlink=None):
+        """Create a symbolic link at `newlink`, pointing here.
 
-    if hasattr(os, 'symlink'):
+        If newlink is not supplied, the symbolic link will assume
+        the name self.basename(), creating the link in the cwd.
 
-        def symlink(self, newlink=None):
-            """Create a symbolic link at `newlink`, pointing here.
+        .. seealso:: :func:`os.symlink`
+        """
+        if newlink is None:
+            newlink = self.basename()
+        os.symlink(self, newlink)
+        return self._next_class(newlink)
 
-            If newlink is not supplied, the symbolic link will assume
-            the name self.basename(), creating the link in the cwd.
+    def readlink(self):
+        """Return the path to which this symbolic link points.
 
-            .. seealso:: :func:`os.symlink`
-            """
-            if newlink is None:
-                newlink = self.basename()
-            os.symlink(self, newlink)
-            return self._next_class(newlink)
+        The result may be an absolute or a relative path.
 
-    if hasattr(os, 'readlink'):
+        .. seealso:: :meth:`readlinkabs`, :func:`os.readlink`
+        """
+        return self._next_class(os.readlink(self))
 
-        def readlink(self):
-            """Return the path to which this symbolic link points.
+    def readlinkabs(self):
+        """Return the path to which this symbolic link points.
 
-            The result may be an absolute or a relative path.
+        The result is always an absolute path.
 
-            .. seealso:: :meth:`readlinkabs`, :func:`os.readlink`
-            """
-            return self._next_class(os.readlink(self))
-
-        def readlinkabs(self):
-            """Return the path to which this symbolic link points.
-
-            The result is always an absolute path.
-
-            .. seealso:: :meth:`readlink`, :func:`os.readlink`
-            """
-            p = self.readlink()
-            if p.isabs():
-                return p
-            else:
-                return (self.parent / p).abspath()
+        .. seealso:: :meth:`readlink`, :func:`os.readlink`
+        """
+        p = self.readlink()
+        return p if p.isabs() else (self.parent / p).abspath()
 
     # High-level functions from shutil
     # These functions will be bound to the instance such that
@@ -1279,7 +1275,7 @@ class Path(str):
         return self
 
     def chdir(self):
-        """ .. seealso:: :func:`os.chdir` """
+        """.. seealso:: :func:`os.chdir`"""
         os.chdir(self)
 
     cd = chdir
@@ -1336,15 +1332,15 @@ class Path(str):
 
     if hasattr(os, 'chroot'):
 
-        def chroot(self):
-            """ .. seealso:: :func:`os.chroot` """
+        def chroot(self):  # pragma: nocover
+            """.. seealso:: :func:`os.chroot`"""
             os.chroot(self)
 
     if hasattr(os, 'startfile'):
 
-        def startfile(self):
-            """ .. seealso:: :func:`os.startfile` """
-            os.startfile(self)
+        def startfile(self, *args, **kwargs):  # pragma: nocover
+            """.. seealso:: :func:`os.startfile`"""
+            os.startfile(self, *args, **kwargs)
             return self
 
     # in-place re-writing, courtesy of Martijn Pieters
@@ -1414,8 +1410,7 @@ class Path(str):
             )
         else:
             os_mode = os.O_CREAT | os.O_WRONLY | os.O_TRUNC
-            if hasattr(os, 'O_BINARY'):
-                os_mode |= os.O_BINARY
+            os_mode |= getattr(os, 'O_BINARY', 0)
             fd = os.open(self, os_mode, perm)
             writable = io.open(
                 fd,
@@ -1623,22 +1618,17 @@ class TempDir(Path):
         return self._next_class(self)
 
     def __exit__(self, exc_type, exc_value, traceback):
-        if not exc_value:
-            self.rmtree()
+        self.rmtree()
 
 
 def _multi_permission_mask(mode):
     """
     Support multiple, comma-separated Unix chmod symbolic modes.
 
-    >>> _multi_permission_mask('a=r,u+w')(0) == 0o644
-    True
+    >>> oct(_multi_permission_mask('a=r,u+w')(0))
+    '0o644'
     """
-
-    def compose(f, g):
-        return lambda *args, **kwargs: g(f(*args, **kwargs))
-
-    return functools.reduce(compose, map(_permission_mask, mode.split(',')))
+    return compose(*map(_permission_mask, reversed(mode.split(','))))
 
 
 def _permission_mask(mode):
@@ -1667,6 +1657,10 @@ def _permission_mask(mode):
 
     >>> _permission_mask('g=')(0o157) == 0o107
     True
+
+    >>> _permission_mask('gobbledeegook')
+    Traceback (most recent call last):
+    ValueError: ('Unrecognized symbolic mode', 'gobbledeegook')
     """
     # parse the symbolic mode
     parsed = re.match('(?P<who>[ugoa]+)(?P<op>[-+=])(?P<what>[rwx]*)$', mode)
